using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using MsCrmHelpers.CrmSdk;
using System.Reflection;

namespace MsCrmHelpers
{
    public class CrmDropDownList : DropDownList
    {
        public CrmDropDownList()
        {
            DataTextField = "name";
            DataValueField = "id";
        }

        protected virtual QueryExpression BuildQuery()
        {
            if (String.IsNullOrEmpty(DataMember))
                throw new ArgumentNullException("DataMember");

            return QuickQueries.BuildQuery(DataMember);
        }

        public override void DataBind()
        {
            this.DataSource = CrmData;

            base.DataBind();
        }

        BusinessEntity[] m_CrmData;
        protected BusinessEntity[] CrmData
        {
            get
            {
                if (m_CrmData == null)
                {
                    m_CrmData = GetDataFromCrm();
                }
                return m_CrmData;
            }

        }
        private BusinessEntity[] GetDataFromCrm()
        {
            if (String.IsNullOrEmpty(DataTextField))
                throw new ArgumentNullException("DataTextField");

            if (String.IsNullOrEmpty(DataValueField))
                throw new ArgumentNullException("DataValueField");

            CrmService crmService = CrmServer.HttpContextCurrent.CrmService;

            // Create the request object.
            RetrieveMultipleRequest retrieve = new RetrieveMultipleRequest();

            // Set the properties of the request object.
            retrieve.Query = BuildQuery();

            // Execute the request.
            RetrieveMultipleResponse searchResults = (RetrieveMultipleResponse)crmService.Execute(retrieve);

            return searchResults.BusinessEntityCollection.BusinessEntities;
        }

        /// <summary>
        /// Gets the selected item as a Lookup value
        /// </summary>
        public BusinessEntity SelectedEntity
        {
            get
            {
                ListItem listItem = this.SelectedItem;
                if (listItem == null) return null;

                foreach (BusinessEntity entity in CrmData)
                {
                    object entityValue = GetValue(entity, DataValueField);
                    if (Object.Equals(entityValue.ToString(), listItem.Value))
                        return entity;
                }
                return null;
            }
        }

        private static object GetValue(BusinessEntity entity, string dataValueField)
        {
            Type entityType = entity.GetType();
            PropertyInfo property = entityType.GetProperty(dataValueField);
            object propertyValue = property.GetValue(entity, null);
            return propertyValue;
        }
    }
}
