using System;
using System.Collections.Generic;
using System.Text;
using MsCrmHelpers.CrmSdk;

namespace MsCrmHelpers
{
    public class CrmProductUnitDropDown: CrmDropDownList
    {
        public CrmProductUnitDropDown()
        {
            DataMember = "uom";
            DataValueField = "uomid";
        }

        protected override QueryExpression BuildQuery()
        {
            QueryExpression query = base.BuildQuery();

            // Create the ConditionExpression.
            ConditionExpression condition = new ConditionExpression();
            // Set the condition for the retrieval to be 
            // when the account owner's last name is not Cannon.
            condition.AttributeName = "uomscheduleid";
            condition.Operator = ConditionOperator.NotEqual;
            condition.Values = new string[] { "Cannon" };
            
            // Build the filter based on the condition.
            FilterExpression filter = new FilterExpression();
            filter.FilterOperator = LogicalOperator.And;
            filter.Conditions = new ConditionExpression[] { condition };

            query.Criteria = filter;

            return query;
        }

        public override void DataBind()
        {
            if (String.IsNullOrEmpty(DataTextField))
                throw new ArgumentNullException("DataTextField");

            if (String.IsNullOrEmpty(DataValueField))
                throw new ArgumentNullException("DataValueField");

            if (String.IsNullOrEmpty(DataMember))
                throw new ArgumentNullException("DataMember");

            CrmService crmService = CrmServer.HttpContextCurrent.CrmService;

            // Create the query.
            QueryExpression query = new QueryExpression();

            // Set the query's properties.
            query.EntityName = DataMember;
            query.ColumnSet = new AllColumns();

            // Create the request object.
            RetrieveMultipleRequest retrieve = new RetrieveMultipleRequest();

            // Set the properties of the request object.
            retrieve.Query = query;

            // Execute the request.
            RetrieveMultipleResponse searchResults = (RetrieveMultipleResponse)crmService.Execute(retrieve);

            this.DataSource = searchResults.BusinessEntityCollection.BusinessEntities;

            base.DataBind();
        }

    }
}
