using System;
using System.Data;
using System.Configuration;

using MsCrmHelpers.CrmSdk;
using System.Net;
using MsCrmHelpers.CrmMetadataService;
using System.Web;

namespace MsCrmHelpers
{
    /// <summary>
    /// CrmServer is a wrapper around the CRM web services
    /// </summary>
    public class CrmServer
    {
        /// <summary>
        /// Gets entity metadata.  Could support aggressive caching in future.
        /// </summary>
        /// <param name="entityName"></param>
        /// <returns></returns>
        public EntityMetadata RetrieveEntityMetadata(string entityName)
        {
            //foreach (EntityMetadata e in this.m_CrmMetadata.Entities) { System.Diagnostics.Debug.WriteLine(e.Name); }
            return MetadataService.RetrieveEntityMetadata(entityName, EntityFlags.All);
        }


        Metadata m_CrmMetadata;
       /// <summary>
       /// The basic entity metadata (list of entities).
       /// Could add aggressive caching.
       /// </summary>
        public Metadata CrmMetadata
        {
            get
            {
                if (m_CrmMetadata == null)
                    m_CrmMetadata = MetadataService.RetrieveMetadata(MetadataFlags.EntitiesOnly);
                return m_CrmMetadata;
            }
        }

        static CrmServer s_Current;


        /// <summary>
        /// Gets the CrmServer stored in the HttpContext, or constructs a
        /// new one if it does not exist.  Security is enforced beause each
        /// request gets its own HttpContext
        /// </summary>
        public static CrmServer Current
        {
            get
            {
                HttpContext context = HttpContext.Current;
                if (context != null) throw new InvalidOperationException();
                
                if (s_Current == null)
                {
                    s_Current = new CrmServer();
                }
                return s_Current;
            }
        }

        /// <summary>
        /// Gets the CrmServer stored in the HttpContext, or constructs a
        /// new one if it does not exist.  Security is enforced beause each
        /// request gets its own HttpContext
        /// </summary>
        public static CrmServer HttpContextCurrent
        {
            get
            {
                HttpContext context = HttpContext.Current;
                if (context == null)
                    throw new InvalidOperationException("No HttpContext");
                CrmServer server = (CrmServer) context.Items[HttpContextCacheKey];
                if (server == null)
                {
                    server = new CrmServer();
                    context.Items.Add(HttpContextCacheKey, server);
                }
                return server;
            }
        }

        private static object HttpContextCacheKey = new object();

        private CrmService ConnectCrmService()
        {
            CrmService service = new CrmService();
            //service.Url = ??;
            //service.Credentials = ??;
            service.Credentials = CredentialCache.DefaultCredentials;

            // CRM always need auth, this speeds it up
            service.PreAuthenticate = true;

            return service;
        }

        private MetadataService ConnectMetadataService()
        {
            MetadataService service = new MetadataService();
            //service.Url = ??;
            //service.Credentials = ??;
            service.Credentials = CredentialCache.DefaultCredentials;

            // CRM always need auth, this speeds it up
            service.PreAuthenticate = true;

            return service;
        }

        private CrmService m_CrmService;
        public CrmService CrmService
        {
            get
            {
                if (m_CrmService == null)
                    m_CrmService = ConnectCrmService();
                return m_CrmService;
            }
        }

        private MetadataService m_MetadataService;
        public MetadataService MetadataService
        {
            get
            {
                if (m_MetadataService == null)
                    m_MetadataService = ConnectMetadataService();
                return m_MetadataService;
            }
        }

        public CrmServer()
        {
        }


        public DynamicEntity GetDynamicEntity(string entityName, Guid id)
        {
            return QuickQueries.GetDynamicEntity(CrmService, entityName, id);
        }
    }
}