using System;
using System.Collections.Generic;
using System.Text;
using MsCrmHelpers.CrmSdk;
using System.Web.Services.Protocols;

namespace MsCrmHelpers
{
    /// <summary>
    ///  A cache for DynamicEntities, meaning that they are only fetched once
    /// </summary>
    public class EntityCache
    {
        private CrmServer m_CrmServer;
        public CrmServer CrmServer
        {
            get { return m_CrmServer; }
        }

        public EntityCache(CrmServer server)
        {
            m_CrmServer = server;
        }

        Dictionary<string, Dictionary<Guid, DynamicEntity>> m_Cache = new Dictionary<string, Dictionary<Guid, DynamicEntity>>();
        /// <summary>
        /// Gets the specified entity, using caching
        /// </summary>
        /// <param name="entityName"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        public DynamicEntity GetEntity(string entityName, Guid id)
        {
            Dictionary<Guid, DynamicEntity> entityCache = null;

            lock (m_Cache)
            {
                if (m_Cache.ContainsKey(entityName))
                {
                    entityCache = m_Cache[entityName];
                }
                else
                {
                    entityCache = new Dictionary<Guid, DynamicEntity>();
                    m_Cache[entityName] = entityCache;
                }
            }

            DynamicEntity entity = null;
            lock (entityCache)
            {
                if (entityCache.ContainsKey(id))
                {
                    entity = entityCache[id];
                }
                else
                {
                    try
                    {
                        entity = CrmServer.GetDynamicEntity(entityName, id);
                    }
                    catch (SoapException soapException)
                    {
                        uint errorCode = SoapExceptionParser.ExtractExceptionCode(soapException);
                        if (errorCode == SoapExceptionParser.ObjectNotFound)
                        {
                            // Do nothing
                        }
                        else
                        {
                            throw;
                        }
                    } 
                    entityCache[id] = entity;
                }
            }

            return entity;
        }

    }
}
