using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace MsCrmHelpers
{
    namespace CrmMetadataService
    {
        public partial class EntityMetadata
        {
            /// <summary>
            /// Locates an item in the attributeMetadata collection
            /// </summary>
            /// <param name="name"></param>
            /// <returns></returns>
            public AttributeMetadata FindAttributeMetadata(string name)
            {
                foreach (AttributeMetadata attributeMetadata in Attributes)
                {
                    if (attributeMetadata.Name.Equals(name, StringComparison.InvariantCultureIgnoreCase)) return attributeMetadata;
                }
                return null;
            }
        }

        public partial class Metadata
        {
            public EntityMetadata FindEntity(string entityName)
            {
                foreach (EntityMetadata entityMetadata in this.Entities)
                {
                    if (entityMetadata.Name != entityName) continue;
                    return entityMetadata;
                }
                return null;
            }


            public string MapTypeCodeToEntityName(int typeCode)
            {
                foreach (EntityMetadata entityMetadata in this.Entities)
                {
                    if (entityMetadata.ObjectTypeCode != typeCode) continue;
                    return entityMetadata.Name;
                }
                throw new ArgumentOutOfRangeException("typeCode", typeCode, "Type code not found");
            }

            public int MapEntityNameToTypeCode(string entityName)
            {
                EntityMetadata entityMetadata = FindEntity(entityName);
                if (entityMetadata == null)
                    throw new ArgumentOutOfRangeException("entityName", entityName, "Entity not found");

                return entityMetadata.ObjectTypeCode;
            }
        }

        public partial class StateAttributeMetadata
        {
            /// <summary>
            /// Locates an option in the Option collection
            /// </summary>
            /// <param name="description"></param>
            /// <returns></returns>
            public StateOption FindStateOption(string description)
            {
                foreach (StateOption stateOption in Options)
                {
                    if (stateOption.Description == description)
                    {
                        return stateOption;
                    }
                }
                return null;
            }
        }


    }
}
