
// =====================================================================
//  File:		ColumnSetHelper.cs
//  Summary:	[FILE DESCRIPTION GOES HERE]
// =====================================================================
//
//  This file is part of the Microsoft CRM 3.0 SDK Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================

using System;
using MsCrmHelpers.CrmSdk;
using Microsoft.Crm.Sdk.Helpers.Collection;

namespace Microsoft.Crm.Sdk.Helpers
{
	public class ColumnSetHelper
	{
		// Fields
		private ColumnsCollection	_attributes;
		private bool				_allColumns;

		#region Properties
		/// <summary>
		/// A collection that represents the attributes and columns the query will return.
		/// </summary>
		public ColumnsCollection Attributes
		{
			get
			{
				return _attributes;
			}
		}

		internal bool AllColumns
		{
			get
			{
				return _allColumns;
			}
		}
		#endregion

		#region Constructor
		/// <summary>
		/// Creates a ColumnSetHelper object that will be used to determine which columns will be returned by the query.
		/// </summary>
		public ColumnSetHelper()
		{
			_attributes = new ColumnsCollection();
		}
		#endregion

		#region Methods

		/// <summary>
		/// Adds a column to the list of attributes that will be returned by the query.
		/// </summary>
		/// <param name="attributeName">Specifies which attribute will be returned.</param>
		public void AddColumn(string attributeName)
		{
			// "Unchecks" all columns.
			if (_allColumns)
			{
				_allColumns = false;
			}

			_attributes.AddColumn(attributeName);
		}

		/// <summary>
		/// Adds a set of columns to the list of attributes that will be returned by the query.
		/// </summary>
		/// <param name="attributeNames">Specifies which attributes will be returned.</param>
		public void AddColumns(params string[] attributeNames)
		{
			foreach (string attributeName in attributeNames)
			{
				this.AddColumn(attributeName);	
			}
		}

		/// <summary>
		/// Specifies that the query will return all attributes for an entity record.
		/// </summary>
		public void AddAllColumns()
		{
			// Enables all columns, although the columns in this helper class are preserved.
			_allColumns = true;
		}

		internal string[] ToStringArray()
		{
			return _attributes.ToStringArray();
		}
		#endregion
	}
}
