
// =====================================================================
//  File:		ConditionExpressionHelper.cs
//  Summary:	[FILE DESCRIPTION GOES HERE]
// =====================================================================
//
//  This file is part of the Microsoft CRM 3.0 SDK Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================

using System;
using MsCrmHelpers.CrmSdk;
using Microsoft.Crm.Sdk.Helpers.Collection;

namespace Microsoft.Crm.Sdk.Helpers
{
	public class ConditionExpressionHelper
	{
		// Fields
		internal ConditionExpression _conditionExpression;
		
		#region Properties
		/// <summary>
		/// Specifies which attribute the condition will be applied to.
		/// </summary>
		public string AttributeName
		{
			get
			{
				return _conditionExpression.AttributeName;
			}
			set
			{
				_conditionExpression.AttributeName = value;
			}
		}

		/// <summary>
		/// A ConditionOperator enum that specifies which operation will be applied to the specified attribute.
		/// </summary>
		public ConditionOperator ConditionOperator
		{
			get
			{
				return _conditionExpression.Operator;
			}
			set
			{
				_conditionExpression.Operator = value;
			}
		}

		/// <summary>
		/// Optional (depending on the specified ConditionOperator) that will specify what parameters the ConditionOperator will use.
		/// </summary>
		/// <remarks>All values will be cast as an object.</remarks>
		public object[] Values
		{
			get
			{
				return _conditionExpression.Values;
			}
		}
		#endregion

		#region Constructors
		/// <summary>
		/// Used to aid in creating a condition for a filter.
		/// </summary>
		public ConditionExpressionHelper()
		{
			_conditionExpression = new ConditionExpression();
		}

		/// <summary>
		/// Used to aid in creating a condition for a filter.
		/// </summary>
		/// <param name="attributeName">Specifies which attribute the condition will be applied to.</param>
		/// <param name="conditionOperator">Specifies which condition will be applied</param>
		public ConditionExpressionHelper(string attributeName, ConditionOperator conditionOperator) : this()
		{
			_conditionExpression.AttributeName	= attributeName;
			_conditionExpression.Operator		= conditionOperator;
		}

		/// <summary>
		/// Used to aid in creating a condition for a filter
		/// </summary>
		/// <param name="attributeName">Specifies which attribute the condition will be applied to.</param>
		/// <param name="conditionOperator">Specifies which condition will be applied.</param>
		/// <param name="value">Specifies what parameter the condition will use to apply to the specified attribute.</param>
		public ConditionExpressionHelper( string attributeName, ConditionOperator conditionOperator, object value) : this(attributeName, conditionOperator)
		{
			_conditionExpression.Values			= new object[] {value};
		}

		/// <summary>
		/// Used to aid in creating a condition for a filter.
		/// </summary>
		/// <param name="attributeName">Specifies which attribute the condition will be applied to.</param>
		/// <param name="conditionOperator">Specifies which condition will be applied.</param>
		/// <param name="values">Specifies what parameters the condition will use to apply to the specified attribute.</param>
		public ConditionExpressionHelper(string attributeName, ConditionOperator conditionOperator, object[] values) : this(attributeName, conditionOperator)
		{
			_conditionExpression.Values			= values;
		}
		#endregion
	}
}
