
// =====================================================================
//  File:		FilterExpressionHelper.cs
//  Summary:	[FILE DESCRIPTION GOES HERE]
// =====================================================================
//
//  This file is part of the Microsoft CRM 3.0 SDK Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================

using System;
using MsCrmHelpers.CrmSdk;
using Microsoft.Crm.Sdk.Helpers.Collection;

namespace Microsoft.Crm.Sdk.Helpers
{
	public class FilterExpressionHelper
	{
		// Fields
		private LogicalOperator						_filterOperator;
		private ConditionExpressionHelperCollection	_conditions;
		private FilterExpressionHelperCollection	_filters;


		#region Properties
		/// <summary>
		/// Specifies what LogicalOperator enum will be used to evaluate the conditions.
		/// </summary>
		/// <remarks>Defaults to 'And'.</remarks>
		public LogicalOperator FilterOperator
		{
			get
			{
				return _filterOperator;
			}
			set
			{
				_filterOperator = value;
			}
		}

		/// <summary>
		/// A collection of conditions that will be used by the filter in the query.
		/// </summary>
		public ConditionExpressionHelperCollection Conditions
		{
			get
			{
				return _conditions;
			}
		}

		/// <summary>
		/// A nested set of child filters.
		/// </summary>
		public FilterExpressionHelperCollection Filters
		{
			get
			{
				return _filters;
			}
		}
		#endregion

		#region Constructor
		/// <summary>
		/// Used to aid in the creation of a filter.
		/// </summary>
		public FilterExpressionHelper()
		{
			_filterOperator	= LogicalOperator.And;
			_conditions		= new ConditionExpressionHelperCollection();
			_filters		= new FilterExpressionHelperCollection();
		}
		#endregion
	}
}
