
// =====================================================================
//  File:		LinkEntityHelper.cs
//  Summary:	[FILE DESCRIPTION GOES HERE]
// =====================================================================
//
//  This file is part of the Microsoft CRM 3.0 SDK Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================

using System;
using MsCrmHelpers.CrmSdk;
using Microsoft.Crm.Sdk.Helpers.Collection;

namespace Microsoft.Crm.Sdk.Helpers
{
	public class LinkEntityHelper
	{
		#region Fields
		private string						_fromEntityName;
		private string						_fromAttributeName;
		private string						_toEntityName;
		private string						_toAttributeName;
		private JoinOperator				_joinOperator;
		private FilterExpressionHelper		_criteria;
		private LinkEntityHelperCollection	_linkEntities;
		#endregion

		#region Properties
		/// <summary>
		/// The entity that is being linked from in the query.
		/// </summary>
		/// <remarks>In the simple case, this is generally the main entity.</remarks>
		public string FromEntityName
		{
			get
			{
				return _fromEntityName;
			}
		}
         		
		/// <summary>
		/// The attribute that is being linked from in the FromEntityName entity.
		/// </summary>
		public string FromAttributeName
		{
			get
			{
				return _fromAttributeName;
			}
		}

		/// <summary>
		/// The entity that is being linked to in the query.
		/// </summary>
		public string ToEntityName
		{
			get
			{
				return _toEntityName;
			}
		}

		/// <summary>
		/// The attribute that is being linked to in the ToEntityName entity (the "linked" entity).
		/// </summary>
		public string ToAttributeName
		{
			get
			{
				return _toAttributeName;
			}
		}

		/// <summary>
		/// A built-in JoinOperator enum that specifies what kind of join will be applied to the main entity and linked entity.
		/// </summary>
		public JoinOperator JoinOperator
		{
			get
			{
				return _joinOperator;
			}
		}

		/// <summary>
		/// A filter that will be applied to the linked entity.
		/// </summary>
		public FilterExpressionHelper Criteria
		{
			get
			{
				return _criteria;
			}
			set
			{
				_criteria = value;
			}
		}

		/// <summary>
		/// A set of child linked entities that links to the linked entity (ToLinkEntity).
		/// </summary>
		public LinkEntityHelperCollection LinkEntities
		{
			get
			{
				return _linkEntities;
			}
		}
		#endregion

		#region Constructor
		/// <summary>
		/// Used to aid linking entities in a query.
		/// </summary>
		/// <param name="fromEntityName">The entity that is being linked from.</param>
		/// <param name="fromAttributeName">The attribute that is being linked from in the FromEntityName entity.</param>
		/// <param name="toEntityName">The entity that is being linked to.</param>
		/// <param name="toAttributeName">The attribute that is being linked to in the ToEntityName entity.</param>
		/// <param name="joinOperator">Specifies what kind of join will happen between the FromEntityName entity and ToEntityName entity.</param>
		public LinkEntityHelper(string fromEntityName, string fromAttributeName, string toEntityName, string toAttributeName, JoinOperator joinOperator)
		{
			_fromEntityName		= fromEntityName;
			_fromAttributeName	= fromAttributeName;
			_toEntityName		= toEntityName;
			_toAttributeName	= toAttributeName;
			_joinOperator		= joinOperator;

			_criteria			= new FilterExpressionHelper();
			_linkEntities		= new LinkEntityHelperCollection(_toEntityName);
		}
		#endregion
	}
}
