using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Crm.Sdk.Helpers;
using MsCrmHelpers.CrmSdk;
using System.Xml;

namespace MsCrmHelpers
{
    public class RichQueryExpression : QueryExpressionHelper
    {
        public RichQueryExpression(EntityName name)
            : base(name)
        {
            Columns.AddAllColumns();
        }

        public string Execute(CrmService crmService)
        {
            if (crmService == null)
                throw new ArgumentNullException("crmService");

            string result = crmService.Fetch(GetRichFetchXml(crmService));
            return result;
        }

        /// <summary>
        /// Gets the fetch XML, without any extra columns
        /// </summary>
        /// <param name="crmService"></param>
        /// <returns></returns>
        public string GetBasicFetchXml(CrmService crmService)
        {
            if (crmService == null)
                throw new ArgumentNullException("crmService");

            // Create the request object.
            QueryExpressionToFetchXmlRequest request = new QueryExpressionToFetchXmlRequest();
            request.Query = this.Query;

            // Execute the request.
            QueryExpressionToFetchXmlResponse response = (QueryExpressionToFetchXmlResponse)crmService.Execute(request);
            return response.FetchXml;
        }

        /// <summary>
        /// Gets the fetch xml, with extra columns on linked tables
        /// </summary>
        /// <param name="crmService"></param>
        /// <returns></returns>
        public string GetRichFetchXml(CrmService crmService)
        {
            if (crmService == null)
                throw new ArgumentNullException("crmService");

            string rawFetchXml = GetBasicFetchXml(crmService);
            XmlDocument fetchXmlDocument = new XmlDocument();
            fetchXmlDocument.LoadXml(rawFetchXml);

            AddLinkEntityAttributes(fetchXmlDocument);

            return fetchXmlDocument.OuterXml;
        }

        /// <summary>
        /// Adds linked columns to the fetchXml request
        /// </summary>
        /// <param name="fetchXmlDocument"></param>
        private void AddLinkEntityAttributes(XmlDocument fetchXmlDocument)
        {
            foreach (LinkEntityHelper linkEntity in base.LinkEntities)
            {
                RichLinkEntity richLinkEntity = linkEntity as RichLinkEntity;
                if (richLinkEntity == null) continue;

                /*
                 * <fetch mapping="logical"> 
  <entity name="salesorder"> 
    <attribute name="name"/> 
    <link-entity name="salesorderdetail" from="salesorderid" 
to="salesorderid"> 
      <attribute name="productid"/> 
    </link-entity> 
  </entity> 
</fetch> */

                string linkNodeXpath = String.Format("/fetch/entity[@name='{0}']/link-entity[@name='{1}'][@from='{2}'][@to='{3}']",
                    richLinkEntity.FromEntityName,
                    richLinkEntity.ToEntityName,
                    richLinkEntity.FromAttributeName,
                    richLinkEntity.ToAttributeName);
                XmlNode linkNode = fetchXmlDocument.SelectSingleNode(linkNodeXpath);
                if (linkNode == null)
                    throw new ApplicationException("Internal error - could not interpret FetchXML request");

                if (richLinkEntity.ColumnSet.AllColumns)
                {
                    // Not sure how to implement this (and it probably isn't a great idea anyway...)
                    throw new NotImplementedException();
                }
                else
                {
                    string[] attributes = richLinkEntity.ColumnSet.ToStringArray();
                    if (attributes.Length == 0) continue;

                    foreach (string attribute in attributes)
                    {
                        XmlElement fetchAttributeNode = linkNode.OwnerDocument.CreateElement("attribute");
                        XmlAttribute fetchAttributeNameNode = linkNode.OwnerDocument.CreateAttribute("name");
                        fetchAttributeNameNode.Value = attribute;

                        fetchAttributeNode.Attributes.Append(fetchAttributeNameNode);

                        linkNode.AppendChild(fetchAttributeNode);
                    }
                }
            }
        }

        /// <summary>
        /// Executes query and returns as XmlDocument
        /// </summary>
        /// <param name="crmService"></param>
        /// <returns></returns>
        public XmlDocument ExecuteGetAsXml(CrmService crmService)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(Execute(crmService));
            return doc;
        }
    }
}
