using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using MsCrmHelpers.CrmSdk;
using MsCrmHelpers.CrmMetadataService;
using MsCrmHelpers;

namespace FastSqlSync
{
    public class FieldMapCollection: List<FieldMap>
    {
        internal void CopySqlToCrm(DataRow sqlRow, EntityMetadata crmEntityMetadata, DynamicEntity crmObject, bool alreadyExists, EntityCache entityCache)
        {
            CrmPropertyList properties = new CrmPropertyList(crmObject.Properties);
            
            foreach (FieldMap fieldMap in this)
            {
                fieldMap.CopySqlToCrm(sqlRow, crmEntityMetadata, properties, alreadyExists, entityCache);
            }

            crmObject.Properties = properties.ToArray();
        }

        /// <summary>
        /// Builds a SQL query that compares the CRM view to the SQL table, to detect changed rows
        /// </summary>
        /// <param name="crmTableName"></param>
        /// <param name="sqlTableName"></param>
        /// <returns></returns>
        internal string ConstructSqlDifferenceClause(EntityMetadata entityMetadata, string crmTableName, string sqlTableName)
        {
            StringBuilder builder = new StringBuilder();
            foreach (FieldMap fieldMap in this)
            {
                if (builder.Length != 0)
                    builder.Append(" OR ");

                builder.Append(fieldMap.ConstructSqlDifferenceClause(entityMetadata, crmTableName, sqlTableName));
            }

            return "(" + builder.ToString() + ")";
        }

    }
}
