using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Data;
using MsCrmHelpers.CrmSdk;
using System.Web.Services.Protocols;
using MsCrmHelpers;
using MsCrmHelpers.CrmMetadataService;
using System.Xml.Serialization;

namespace FastSqlSync
{
    /// <summary>
    /// Maps a table into a CRM table
    /// </summary>
    public class ImportTableIntoCrm
    {
        private bool m_ForceUpdate;
        public bool ForceUpdate
        {
            get { return m_ForceUpdate; }
            set { m_ForceUpdate = value; }
        }
	
        private FieldMapCollection m_Fields;

        public FieldMapCollection Fields
        {
            get { return m_Fields; }
            set { m_Fields = value; }
        }

        private SqlConnection m_SqlConnection;
        [XmlIgnore]
        public SqlConnection SqlConnection
        {
            get { return m_SqlConnection; }
            set { m_SqlConnection = value; }
        }

        private DatabaseTable m_CrmTable;
        public DatabaseTable CrmTable
        {
            get { return m_CrmTable; }
            set { m_CrmTable = value; }
        }

        private DatabaseTable m_SqlTable;
        public DatabaseTable SqlTable
        {
            get { return m_SqlTable; }
            set { m_SqlTable = value; }
        }

        public ImportTableIntoCrm()
        {
        }




        #region SQL Queries

        private string ReplaceMacros(string template)
        {
            string expanded = template;

            expanded = expanded.Replace("[SqlTableName]", SqlTable.FullyQualifiedName);
            expanded = expanded.Replace("[CrmTableName]", CrmTable.FullyQualifiedName);

            expanded = expanded.Replace("[SqlTableId]", "[" + SqlTable.IdColumn + "]");
            expanded = expanded.Replace("[CrmTableId]", "[" + CrmTable.IdColumn + "]");

            expanded = expanded.Replace("[ExistsInCrm]", FieldMap.SpecialColumnExistsInCrm);

            return expanded;
        }

        public string SqlSelectRowsNotInCrm
        {
            get
            {
                string sqlSelectRowsNotInCrm =
                    "SELECT * FROM [SqlTableName] " +
                " WHERE ([SqlTableId] NOT IN (SELECT [CrmTableId] FROM [CrmTableName]))";

                switch (SqlTable.Direction)
                {
                    case DatabaseTable.SynchonizeDirection.Source:
                        // Do nothing
                        break;

                    default:
                        //" AND (ISNULL([ExistsInCrm],0) = 0)";
                        throw new NotImplementedException();
                }


                return ReplaceMacros(sqlSelectRowsNotInCrm);
            }
        }

        public string SqlSelectRowsAlreadyInCrm
        {
            get
            {
                //string sqlSelectRowsAlreadyInCrm =
                //    "SELECT * FROM [SqlTableName] " +
                //" WHERE ([SqlTableId] IN (SELECT [CrmTableId] FROM [CrmTableName]))";

                string sqlSelectRowsAlreadyInCrm =
                    "SELECT [SqlTableName].* FROM [SqlTableName] " +
                " JOIN [CrmTableName] ON [SqlTableId] = [CrmTableId]";

                if (!ForceUpdate)
                {
                    string different = Fields.ConstructSqlDifferenceClause(CrmEntityMetadata, CrmTable.FullyQualifiedName, SqlTable.FullyQualifiedName);

                    sqlSelectRowsAlreadyInCrm += " WHERE " + different;
                }

                switch (SqlTable.Direction)
                {
                    case DatabaseTable.SynchonizeDirection.Source:
                        // Do nothing
                        break;

                    default:
                        //" AND (ISNULL([ExistsInCrm],0) = 0)";
                        throw new NotImplementedException();
                }


                return ReplaceMacros(sqlSelectRowsAlreadyInCrm);
            }
        }
        #endregion


        public void DoPostReplicationSync()
        {
            ProcessRowsNotInCrm();

            ReProcessRowsAlreadyInCrm();
        }


        private void ProcessRowsNotInCrm()
        {
            DataSet insertedDataSet = new DataSet();

            using (SqlDataAdapter adapter = new SqlDataAdapter(SqlSelectRowsNotInCrm, SqlConnection))
            {
                adapter.Fill(insertedDataSet);
            }

            foreach (DataRow row in insertedDataSet.Tables[0].Rows)
            {
                ProcessRow(row, false);
            }
        }

        private void ReProcessRowsAlreadyInCrm()
        {
            DataSet updatedDataSet = new DataSet();

            using (SqlDataAdapter adapter = new SqlDataAdapter(SqlSelectRowsAlreadyInCrm, SqlConnection))
            {
                adapter.SelectCommand.CommandTimeout = 300;
                adapter.Fill(updatedDataSet);
            }

            foreach (DataRow row in updatedDataSet.Tables[0].Rows)
            {
                ProcessRow(row, true);
            }
        }

        EntityMetadata m_CrmEntityMetadata;
        public EntityMetadata CrmEntityMetadata
        {
            get
            {
                if (m_CrmEntityMetadata == null)
                    m_CrmEntityMetadata = CrmServer.Current.RetrieveEntityMetadata(CrmTable.EntityName);
                return m_CrmEntityMetadata;
            }
        }

        private EntityCache m_EntityCache;
        public EntityCache EntityCache
        {
            get
            {
                if (m_EntityCache == null)
                {
                    m_EntityCache = new EntityCache(CrmServer.Current);
                }
                return m_EntityCache;
            }
        }

        Guid fakeUser = new Guid("64d12d42-cfc9-4c36-ad04-ff2d9f1b0cbd");

        private void ProcessRow(DataRow sqlRow, bool alreadyExists)
        {
            DynamicEntity crmObject;

            if (alreadyExists)
            {
                Guid id = (Guid)sqlRow[SqlTable.IdColumn];
                crmObject = CrmServer.Current.GetDynamicEntity(CrmTable.EntityName, id);
            }
            else
            {
                crmObject = new DynamicEntity();
                crmObject.Name = CrmTable.EntityName;
            }


            try
            {
                Fields.CopySqlToCrm(sqlRow, CrmEntityMetadata, crmObject, alreadyExists, EntityCache);
            }
            catch (ArgumentOutOfRangeException argumentOorException)
            {
                if (argumentOorException.ActualValue.Equals(fakeUser))
                {
                    return;
                }
                ProgressLog.Current.WriteError("Failed to map object to CRM: " + sqlRow);
                ProgressLog.Current.WriteError(argumentOorException.ToString());
                return;
            }
            catch (Exception exception)
            {
                ProgressLog.Current.WriteError("Failed to map object to CRM: " + sqlRow);
                ProgressLog.Current.WriteError(exception.ToString());
                return;
            }

            try
            {
                if (alreadyExists)
                {
                    ProgressLog.Current.WriteInfo("Updating object in CRM: " + crmObject.Name);
                    CrmServer.Current.CrmService.Update(crmObject);
                }
                else
                {
                    ProgressLog.Current.WriteInfo("Creating object in CRM: " + crmObject.Name);
                    Guid newCrmId = CrmServer.Current.CrmService.Create(crmObject);
                }
            }
            catch (SoapException soapException)
            {
                ProgressLog.Current.WriteError("Failed to write object to CRM: " + crmObject.Name);
                foreach (Property property in crmObject.Properties)
                {
                    ProgressLog.Current.WriteInfo("\t" + property.ToString());
                }
                ProgressLog.Current.WriteError(soapException.Detail.OuterXml);
                return;
            }

            //CopyBackSingleObject(newCrmId);
        }
    }
}
