using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using Microsoft.SqlServer.Replication;
using Microsoft.SqlServer.Management.Common;
using System.Diagnostics;
using Microsoft.SqlServer.Management.Smo;

namespace FastSqlSync
{
    /// <summary>
    /// Performs a merge replication push operation, synchronization a publication with all subscribers
    /// </summary>
    public class MergePushReplication
    {
        public string PublicationName;
        
        public MergePushReplication()
        {
        }

        public void PerformMergeReplication(SqlConnection subscriberSqlConnection)
        {
            // Create a connection to the Subscriber.
            ServerConnection subscriberConnection = null;
            try
            {
                //subscriberSqlConnection.Open();
                subscriberConnection = new ServerConnection(subscriberSqlConnection);

                // Connect to the Subscriber.
                subscriberConnection.Connect();

                Server server = new Server(subscriberConnection);
                ReplicationServer replicationServer = server.ReplicationServer;
                ReplicationDatabase replicationDatabase = replicationServer.ReplicationDatabases[subscriberSqlConnection.Database];

                MergePublication publication = replicationDatabase.MergePublications[PublicationName];

                foreach (MergeSubscription subscriber in publication.MergeSubscriptions)
                {
                    // Get the agent for the subscription.
                    MergeSynchronizationAgent agent = subscriber.SynchronizationAgent;

                    // Enable agent output to the console.
                    agent.OutputVerboseLevel = 1;
                    agent.Output = "";

                    agent.Status += new AgentCore.StatusEventHandler(agent_Status);
                    
                    // Synchronously start the Merge Agent for the subscription.
                    agent.Synchronize();
                }
            }
            finally
            {
                if (subscriberConnection != null)
                    subscriberConnection.Disconnect();
            }
        }

        void agent_Status(object sender, StatusEventArgs e)
        {
            switch (e.MessageStatus)
            {
                case MessageStatus.Fail:
                    ProgressLog.Current.WriteError(e.Message);
                    break;

                case MessageStatus.Idle:
                case MessageStatus.InProgress:
                case MessageStatus.None:
                case MessageStatus.Retry:
                case MessageStatus.Start:
                case MessageStatus.Succeed:
                    ProgressLog.Current.WriteInfo(e.Message);
                    break;
            }
        }
    }
}
