using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Data.SqlClient;
using MsCrmHelpers.CrmSdk;
using MsCrmHelpers;
using System.IO;
using System.Xml.Serialization;
using System.Diagnostics;
using System.Text;
using System.Net.Mail;
using System.Configuration;

namespace FastSqlSync
{
    /// <summary>
    /// Entry Point for Program
    /// </summary>
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {

            try
            {
                DirectoryInfo appDirectory = new FileInfo(Application.ExecutablePath).Directory;
                SyncScript script = SyncScript.Deserialize(new FileInfo(Path.Combine(appDirectory.FullName, "crmsync.xml")));

                script.Execute();

                XmlSerializer tableImportSerializer = new XmlSerializer(typeof(ImportTableIntoCrm));
                foreach (FileInfo xmlFile in appDirectory.GetFiles("*.import.xml"))
                {
                    using (StreamReader reader = xmlFile.OpenText())
                    {
                        ImportTableIntoCrm tableImport = (ImportTableIntoCrm)tableImportSerializer.Deserialize(reader);
                        tableImport.SqlConnection = script.TargetConnection;
                        tableImport.DoPostReplicationSync();
                    }
                }
            }
            catch (Exception ex)
            {
                ProgressLog.Current.WriteError("Unexpected exception: " + ex);
            }

            if ((ProgressLog.Current.ErrorCount != 0) || (ProgressLog.Current.WarningCount != 0))
            {
                string emailErrorsTo = ConfigurationManager.AppSettings["EmailErrorsTo"];
                if (String.IsNullOrEmpty(emailErrorsTo))
                {
                    Console.WriteLine(ProgressLog.Current.Log);
                }
                else
                {
                    MailMessage notifyMessage = new MailMessage();
                    notifyMessage.To.Add(emailErrorsTo);
                    notifyMessage.Subject = "CRM Synchronization Problem";

                    StringBuilder body = new StringBuilder();
                    body.AppendLine(String.Format("<b>The CRM synchronization process had {0} errors and {1} warnings<b><br/><br/>", ProgressLog.Current.ErrorCount, ProgressLog.Current.WarningCount));
                    body.AppendLine();
                    body.Append(ProgressLog.Current.Log);
                    notifyMessage.Body = body.ToString();
                    notifyMessage.IsBodyHtml = true;

                    SmtpClient client = new SmtpClient();
                    client.Send(notifyMessage);
                }
            }
        }
    }
}